#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
import numpy as np

from simulariumio.cytosim import (
    CytosimConverter,
    CytosimData,
    CytosimObjectInfo,
)
from simulariumio import MetaData, DisplayData, InputFileData
from simulariumio.constants import (
    DEFAULT_CAMERA_SETTINGS,
    CURRENT_VERSION,
    DISPLAY_TYPE,
)


@pytest.mark.parametrize(
    "trajectory, expected_data",
    [
        # 3 fiber agents
        (
            CytosimData(
                meta_data=MetaData(
                    box_size=np.array([0.5, 0.5, 0.5]),
                    scale_factor=1e3,
                ),
                object_info={
                    "fibers": CytosimObjectInfo(
                        cytosim_file=InputFileData(
                            file_path=(
                                "simulariumio/tests/data/cytosim"
                                "/3_fibers_3_frames/fiber_points.txt"
                            ),
                        ),
                        display_data={
                            0: DisplayData(
                                name="fiber",
                                radius=0.001,
                                display_type=DISPLAY_TYPE.SPHERE,
                                color="#d71f5f",
                            )
                        },
                    )
                },
            ),
            {
                "trajectoryInfo": {
                    "version": CURRENT_VERSION.TRAJECTORY_INFO,
                    "timeUnits": {
                        "magnitude": 1.0,
                        "name": "s",
                    },
                    "timeStepSize": 0.1,
                    "totalSteps": 3,
                    "spatialUnits": {
                        "magnitude": 1.0,
                        "name": "nm",
                    },
                    "size": {"x": 500.0, "y": 500.0, "z": 500.0},
                    "cameraDefault": {
                        "position": {
                            "x": DEFAULT_CAMERA_SETTINGS.CAMERA_POSITION[0],
                            "y": DEFAULT_CAMERA_SETTINGS.CAMERA_POSITION[1],
                            "z": DEFAULT_CAMERA_SETTINGS.CAMERA_POSITION[2],
                        },
                        "lookAtPosition": {
                            "x": DEFAULT_CAMERA_SETTINGS.LOOK_AT_POSITION[0],
                            "y": DEFAULT_CAMERA_SETTINGS.LOOK_AT_POSITION[1],
                            "z": DEFAULT_CAMERA_SETTINGS.LOOK_AT_POSITION[2],
                        },
                        "upVector": {
                            "x": DEFAULT_CAMERA_SETTINGS.UP_VECTOR[0],
                            "y": DEFAULT_CAMERA_SETTINGS.UP_VECTOR[1],
                            "z": DEFAULT_CAMERA_SETTINGS.UP_VECTOR[2],
                        },
                        "fovDegrees": DEFAULT_CAMERA_SETTINGS.FOV_DEGREES,
                    },
                    "typeMapping": {
                        "0": {
                            "name": "fiber",
                            "geometry": {
                                "displayType": "FIBER",
                                "color": "#d71f5f",
                            },
                        },
                    },
                },
                "spatialData": {
                    "version": CURRENT_VERSION.SPATIAL_DATA,
                    "msgType": 1,
                    "bundleStart": 0,
                    "bundleSize": 3,
                    "bundleData": [
                        {
                            "frameNumber": 0,
                            "time": 0.1,
                            "data": [
                                1001.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                9.0,
                                -3.70929,
                                110.164,
                                -400.052,
                                -009.70537,
                                110.184,
                                -391.947,
                                -15.7378,
                                110.304,
                                -383.87,
                                1001.0,
                                2.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                18.0,
                                39.440000000000005,
                                -60.4351,
                                -344.994,
                                37.8824,
                                -50.56,
                                -347.361,
                                36.3195,
                                -40.6802,
                                -349.704,
                                34.7773,
                                -30.804000000000002,
                                -352.077,
                                33.265,
                                -20.9349,
                                -354.497,
                                31.8441,
                                -11.0556,
                                -356.932,
                                1001.0,
                                3.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                12.0,
                                42.215700000000005,
                                -93.7993,
                                -349.962,
                                40.2307,
                                -101.867,
                                -346.47499999999997,
                                38.2648,
                                -109.90700000000001,
                                -342.916,
                                36.3465,
                                -117.922,
                                -339.274,
                            ],
                        },
                        {
                            "frameNumber": 1,
                            "time": 0.2,
                            "data": [
                                1001.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                9.0,
                                -5.69782,
                                129.971,
                                -393.00800000000004,
                                -15.290700000000001,
                                127.437,
                                -394.796,
                                -24.8686,
                                124.766,
                                -396.463,
                                1001.0,
                                2.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                18.0,
                                19.0975,
                                -44.6282,
                                -345.032,
                                028.7449,
                                -41.2174,
                                -345.948,
                                38.4093,
                                -37.8622,
                                -346.891,
                                48.0874,
                                -34.5692,
                                -347.90900000000005,
                                57.7825,
                                -31.335799999999995,
                                -348.95599999999996,
                                67.48859999999999,
                                -28.145300000000002,
                                -350.03299999999996,
                                1001.0,
                                3.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                12.0,
                                -2.25528,
                                -76.97500000000001,
                                -390.275,
                                2.86711,
                                -71.30789999999999,
                                -385.498,
                                7.987640000000001,
                                -65.6548,
                                -380.702,
                                13.1339,
                                -59.9998,
                                -375.936,
                            ],
                        },
                        {
                            "frameNumber": 2,
                            "time": 0.3,
                            "data": [
                                1001.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                9.0,
                                8.850850000000001,
                                142.58399999999997,
                                -381.975,
                                -0.66752,
                                141.898,
                                -378.722,
                                -10.1991,
                                141.17499999999998,
                                -375.517,
                                1001.0,
                                2.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                18.0,
                                -53.0708,
                                -112.578,
                                -323.72499999999997,
                                -48.829299999999996,
                                -106.16799999999999,
                                -330.54200000000003,
                                -44.5379,
                                -99.7703,
                                -337.339,
                                -40.219199999999994,
                                -93.4137,
                                -344.157,
                                -35.9216,
                                -87.07,
                                -351.00100000000003,
                                -31.5842,
                                -80.7395,
                                -357.831,
                                1001.0,
                                3.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                12.0,
                                33.213300000000004,
                                -102.894,
                                -367.733,
                                26.529799999999998,
                                -98.4461,
                                -363.644,
                                19.8645,
                                -93.94890000000001,
                                -359.57800000000003,
                                13.2555,
                                -89.3484,
                                -355.537,
                            ],
                        },
                    ],
                },
                "plotData": {"version": CURRENT_VERSION.PLOT_DATA, "data": []},
            },
        ),
        # aster_pull3D example with couples, actin fibers, and solids added
        (
            CytosimData(
                meta_data=MetaData(
                    box_size=np.array([2.0, 2.0, 2.0]),
                    scale_factor=100.0,
                ),
                object_info={
                    "fibers": CytosimObjectInfo(
                        cytosim_file=InputFileData(
                            file_path=(
                                "simulariumio/tests/data/cytosim/aster_pull3D"
                                "_couples_actin_solid_3_frames/fiber_points.txt"
                            ),
                        ),
                        display_data={
                            2: DisplayData(
                                name="actin",
                                radius=0.01,
                                color="#ffc100",
                            ),
                        },
                    ),
                    "solids": CytosimObjectInfo(
                        cytosim_file=InputFileData(
                            file_path=(
                                "simulariumio/tests/data/cytosim/"
                                "aster_pull3D_couples_actin_solid_3_frames/solids.txt"
                            ),
                        ),
                        display_data={
                            1: DisplayData(name="aster"),
                            2: DisplayData(name="vesicle", radius=0.1),
                        },
                    ),
                    "singles": CytosimObjectInfo(
                        cytosim_file=InputFileData(
                            file_path=(
                                "simulariumio/tests/data/cytosim/"
                                "aster_pull3D_couples_actin_solid_3_frames/singles.txt"
                            ),
                        ),
                        display_data={
                            1: DisplayData(name="dynein", radius=0.01),
                            2: DisplayData(
                                name="kinesin",
                                radius=0.01,
                                display_type=DISPLAY_TYPE.PDB,
                                url="https://files.rcsb.org/download/3KIN.pdb",
                                color="#0080ff",
                            ),
                        },
                    ),
                    "couples": CytosimObjectInfo(
                        cytosim_file=InputFileData(
                            file_path=(
                                "simulariumio/tests/data/cytosim/"
                                "aster_pull3D_couples_actin_solid_3_frames/couples.txt"
                            ),
                        ),
                        display_data={
                            1: DisplayData(name="motor complex", radius=0.02)
                        },
                        position_indices=[3, 4, 5],
                    ),
                },
            ),
            {
                "trajectoryInfo": {
                    "version": CURRENT_VERSION.TRAJECTORY_INFO,
                    "timeUnits": {
                        "magnitude": 1.0,
                        "name": "s",
                    },
                    "timeStepSize": 0.05,
                    "totalSteps": 3,
                    "spatialUnits": {
                        "magnitude": 10.0,
                        "name": "nm",
                    },
                    "size": {"x": 200.0, "y": 200.0, "z": 200.0},
                    "cameraDefault": {
                        "position": {
                            "x": DEFAULT_CAMERA_SETTINGS.CAMERA_POSITION[0],
                            "y": DEFAULT_CAMERA_SETTINGS.CAMERA_POSITION[1],
                            "z": DEFAULT_CAMERA_SETTINGS.CAMERA_POSITION[2],
                        },
                        "lookAtPosition": {
                            "x": DEFAULT_CAMERA_SETTINGS.LOOK_AT_POSITION[0],
                            "y": DEFAULT_CAMERA_SETTINGS.LOOK_AT_POSITION[1],
                            "z": DEFAULT_CAMERA_SETTINGS.LOOK_AT_POSITION[2],
                        },
                        "upVector": {
                            "x": DEFAULT_CAMERA_SETTINGS.UP_VECTOR[0],
                            "y": DEFAULT_CAMERA_SETTINGS.UP_VECTOR[1],
                            "z": DEFAULT_CAMERA_SETTINGS.UP_VECTOR[2],
                        },
                        "fovDegrees": DEFAULT_CAMERA_SETTINGS.FOV_DEGREES,
                    },
                    "typeMapping": {
                        "0": {
                            "name": "fiber1",
                            "geometry": {
                                "displayType": "FIBER",
                            },
                        },
                        "1": {
                            "name": "actin",
                            "geometry": {
                                "displayType": "FIBER",
                                "color": "#ffc100",
                            },
                        },
                        "2": {
                            "name": "aster",
                            "geometry": {
                                "displayType": "SPHERE",
                            },
                        },
                        "3": {
                            "name": "vesicle",
                            "geometry": {
                                "displayType": "SPHERE",
                            },
                        },
                        "4": {
                            "name": "kinesin",
                            "geometry": {
                                "displayType": "PDB",
                                "url": "https://files.rcsb.org/download/3KIN.pdb",
                                "color": "#0080ff",
                            },
                        },
                        "5": {
                            "name": "dynein",
                            "geometry": {
                                "displayType": "SPHERE",
                            },
                        },
                        "6": {
                            "name": "motor complex",
                            "geometry": {
                                "displayType": "SPHERE",
                            },
                        },
                    },
                },
                "spatialData": {
                    "version": CURRENT_VERSION.SPATIAL_DATA,
                    "msgType": 1,
                    "bundleStart": 0,
                    "bundleSize": 3,
                    "bundleData": [
                        {
                            "frameNumber": 0,
                            "time": 0.0,
                            "data": [
                                1001.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                15.0,
                                36.93,
                                36.80,
                                16.78,
                                30.55,
                                43.87,
                                19.84,
                                24.169999999999998,
                                50.93,
                                22.900000000000002,
                                17.78,
                                57.99999999999999,
                                25.95,
                                11.40,
                                65.07,
                                29.01,
                                1001.0,
                                2.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                12.0,
                                36.93,
                                36.80,
                                16.78,
                                37.20,
                                27.11,
                                19.220000000000002,
                                37.46,
                                17.41,
                                21.66,
                                37.730000000000004,
                                7.720000000000001,
                                24.099999999999998,
                                1001.0,
                                3.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                15.0,
                                36.93,
                                36.80,
                                16.78,
                                40.71,
                                30.86,
                                23.880000000000003,
                                44.50,
                                24.92,
                                30.98,
                                48.28,
                                18.970000000000002,
                                38.07,
                                52.07000000000001,
                                13.03,
                                45.17,
                                1001.0,
                                13.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                12.0,
                                46.08,
                                -104.85,
                                -2.10,
                                45.67,
                                -104.54,
                                -1.90,
                                45.26,
                                -104.23,
                                -1.7000000000000002,
                                44.85,
                                -103.91999999999999,
                                -1.50,
                                1001.0,
                                14.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                9.0,
                                53.690000000000005,
                                79.28,
                                -3.09,
                                53.60,
                                78.83,
                                -3.39,
                                53.50,
                                78.39,
                                -3.6999999999999997,
                                1000.0,
                                4.0,
                                2.0,
                                36.93,
                                36.80,
                                16.78,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                0.0,
                                1000.0,
                                5.0,
                                3.0,
                                78.60000000000001,
                                -28.799999999999997,
                                18.04,
                                0.0,
                                0.0,
                                0.0,
                                10.0,
                                0.0,
                                1000.0,
                                6.0,
                                3.0,
                                -55.089999999999996,
                                -33.42,
                                -61.85000000000001,
                                0.0,
                                0.0,
                                0.0,
                                10.0,
                                0.0,
                                1000.0,
                                7.0,
                                4.0,
                                20.61,
                                -39.910000000000004,
                                89.35,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                8.0,
                                4.0,
                                92.82000000000001,
                                32.269999999999996,
                                18.55,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                9.0,
                                4.0,
                                37.56,
                                -80.46,
                                -46.00,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                101.0,
                                5.0,
                                -74.11999999999999,
                                30.31,
                                -59.89,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                102.0,
                                5.0,
                                -61.739999999999995,
                                -55.269999999999996,
                                -55.98,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                103.0,
                                5.0,
                                -19.37,
                                -9.379999999999999,
                                97.66,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                10.0,
                                6.0,
                                32.629999999999995,
                                38.550000000000004,
                                57.120000000000005,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                                1000.0,
                                11.0,
                                6.0,
                                4.42,
                                20.200000000000003,
                                -62.88,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                                1000.0,
                                12.0,
                                6.0,
                                -73.80,
                                -25.20,
                                -43.89,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                            ],
                        },
                        {
                            "frameNumber": 1,
                            "time": 0.05,
                            "data": [
                                1001.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                18.0,
                                44.55,
                                33.97,
                                8.86,
                                36.63,
                                38.269999999999996,
                                4.51,
                                28.96,
                                43.28,
                                0.50,
                                21.83,
                                49.61,
                                -2.52,
                                16.00,
                                57.66,
                                -3.58,
                                12.85,
                                66.92,
                                -1.47,
                                1001.0,
                                2.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                9.0,
                                44.230000000000004,
                                33.56,
                                9.04,
                                42.02,
                                24.04,
                                6.92,
                                40.52,
                                14.330000000000002,
                                5.050000000000001,
                                1001.0,
                                3.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                15.0,
                                44.22,
                                33.18,
                                8.89,
                                40.660000000000004,
                                24.060000000000002,
                                10.94,
                                37.22,
                                15.10,
                                13.750000000000002,
                                34.00,
                                6.74,
                                18.20,
                                31.369999999999997,
                                -0.06999999999999999,
                                25.03,
                                1001.0,
                                13.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                15.0,
                                51.480000000000004,
                                -85.75,
                                5.3100000000000005,
                                51.00,
                                -85.78,
                                5.59,
                                50.529999999999994,
                                -85.82,
                                5.87,
                                50.06,
                                -85.86,
                                6.15,
                                49.58,
                                -85.90,
                                6.419999999999999,
                                1001.0,
                                14.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                9.0,
                                47.00,
                                81.69999999999999,
                                -9.139999999999999,
                                47.099999999999994,
                                81.22,
                                -9.39,
                                47.21,
                                80.74,
                                -9.64,
                                1000.0,
                                4.0,
                                2.0,
                                44.86,
                                34.239999999999995,
                                9.36,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                0.0,
                                1000.0,
                                5.0,
                                3.0,
                                77.18,
                                -36.65,
                                21.90,
                                0.0,
                                0.0,
                                0.0,
                                10.0,
                                0.0,
                                1000.0,
                                6.0,
                                3.0,
                                -59.67,
                                -29.659999999999997,
                                -65.88000000000001,
                                0.0,
                                0.0,
                                0.0,
                                10.0,
                                0.0,
                                1000.0,
                                7.0,
                                4.0,
                                20.61,
                                -39.910000000000004,
                                89.35,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                8.0,
                                4.0,
                                92.82000000000001,
                                32.269999999999996,
                                18.55,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                9.0,
                                4.0,
                                37.56,
                                -80.46,
                                -46.00,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                101.0,
                                5.0,
                                -74.11999999999999,
                                30.31,
                                -59.89,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                102.0,
                                5.0,
                                -61.739999999999995,
                                -55.269999999999996,
                                -55.98,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                103.0,
                                5.0,
                                -19.37,
                                -9.379999999999999,
                                97.66,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                10.0,
                                6.0,
                                33.96,
                                13.309999999999999,
                                50.29,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                                1000.0,
                                11.0,
                                6.0,
                                19.189999999999998,
                                17.69,
                                -76.94,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                                1000.0,
                                12.0,
                                6.0,
                                -72.52,
                                -21.90,
                                -43.59,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                            ],
                        },
                        {
                            "frameNumber": 2,
                            "time": 0.1,
                            "data": [
                                1001.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                15.0,
                                44.55,
                                33.97,
                                8.86,
                                36.63,
                                38.269999999999996,
                                4.51,
                                28.96,
                                43.28,
                                0.50,
                                21.83,
                                49.61,
                                -2.52,
                                16.00,
                                57.66,
                                -3.58,
                                1001.0,
                                2.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                6.0,
                                44.230000000000004,
                                33.56,
                                9.04,
                                42.02,
                                24.04,
                                6.92,
                                1001.0,
                                3.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                12.0,
                                44.22,
                                33.18,
                                8.89,
                                40.660000000000004,
                                24.060000000000002,
                                10.94,
                                37.22,
                                15.10,
                                13.750000000000002,
                                34.00,
                                6.74,
                                18.20,
                                1001.0,
                                13.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                6.0,
                                51.480000000000004,
                                -85.75,
                                5.3100000000000005,
                                51.00,
                                -85.78,
                                5.59,
                                1001.0,
                                14.0,
                                1.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                12.0,
                                47.00,
                                81.69999999999999,
                                -9.139999999999999,
                                47.099999999999994,
                                81.22,
                                -9.39,
                                47.21,
                                80.74,
                                -9.64,
                                47.31,
                                80.25999999999999,
                                -9.89,
                                1000.0,
                                4.0,
                                2.0,
                                44.86,
                                34.239999999999995,
                                9.36,
                                0.0,
                                0.0,
                                0.0,
                                100.0,
                                0.0,
                                1000.0,
                                5.0,
                                3.0,
                                77.18,
                                -36.65,
                                21.90,
                                0.0,
                                0.0,
                                0.0,
                                10.0,
                                0.0,
                                1000.0,
                                6.0,
                                3.0,
                                -59.67,
                                -29.659999999999997,
                                -65.88000000000001,
                                0.0,
                                0.0,
                                0.0,
                                10.0,
                                0.0,
                                1000.0,
                                7.0,
                                4.0,
                                20.61,
                                -39.910000000000004,
                                89.35,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                8.0,
                                4.0,
                                92.82000000000001,
                                32.269999999999996,
                                18.55,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                9.0,
                                4.0,
                                37.56,
                                -80.46,
                                -46.00,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                101.0,
                                5.0,
                                -74.11999999999999,
                                30.31,
                                -59.89,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                102.0,
                                5.0,
                                -61.739999999999995,
                                -55.269999999999996,
                                -55.98,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                103.0,
                                5.0,
                                -19.37,
                                -9.379999999999999,
                                97.66,
                                0.0,
                                0.0,
                                0.0,
                                1.0,
                                0.0,
                                1000.0,
                                10.0,
                                6.0,
                                33.96,
                                13.309999999999999,
                                50.29,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                                1000.0,
                                11.0,
                                6.0,
                                19.189999999999998,
                                17.69,
                                -76.94,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                                1000.0,
                                12.0,
                                6.0,
                                -72.52,
                                -21.90,
                                -43.59,
                                0.0,
                                0.0,
                                0.0,
                                2.0,
                                0.0,
                            ],
                        },
                    ],
                },
                "plotData": {"version": CURRENT_VERSION.PLOT_DATA, "data": []},
            },
        ),
    ],
)
def test_cytosim_converter(trajectory, expected_data):
    converter = CytosimConverter(trajectory)
    buffer_data = converter._read_trajectory_data(converter._data)
    assert expected_data == buffer_data
    assert converter._check_agent_ids_are_unique_per_frame(buffer_data)
